#!/usr/bin/python2
#
# Copyright 2012 Perttu Ahola <celeron55@gmail.com>; All rights reserved.
#

import pygame
import sys
import random
import copy
try:
	import pygame._view # For cx_freeze
except:
	pass

fullscreen = False
if len(sys.argv) == 2 and sys.argv[1] == "fs":
	fullscreen = True

img_path = 'img'
images = {}
colorkey = (0,0,0)

#SCREEN_W = 200
#SCREEN_H = 150
SCREEN_W = 240
SCREEN_H = 180
SHIP_W = 16
SHIP_H = 16

TICK_LENGTH = 1000/30

ship_src_rect = pygame.Rect(0, 0, SHIP_W, SHIP_H)
ship_small_src_rect = pygame.Rect(0, 16, 8, 8)
ship_laser_src_rect = pygame.Rect(32, 0, 16, 16)
explosion_src_rect = pygame.Rect(64, 0, 16, 16)
alien_laser1_src_rect = pygame.Rect(48, 0, 16, 16)
alien1_src_rects = [pygame.Rect(16, 0, 16, 16), pygame.Rect(16, 16, 16, 16)]
alien2_src_rects = [pygame.Rect(80, 0, 16, 16), pygame.Rect(80, 16, 16, 16)]
alien3_src_rects = [pygame.Rect(96, 0, 16, 16), pygame.Rect(96, 16, 16, 16), pygame.Rect(96, 32, 16, 16)]

do_exit = False

def get_image(filename):
	global images
	global img_path
	if filename in images:
		return images[filename]
	#images[filename] = pygame.image.load(img_path + '/' + filename + '.png').convert_alpha()
	images[filename] = pygame.image.load(img_path + '/' + filename + '.png').convert()
	images[filename].set_colorkey(colorkey)
	return images[filename]

def draw_text(screen, pos, text):
	for char in text:
		ic = ord(char)
		iy = ic // 16
		ix = ic - iy * 16
		screen.blit(get_image("font"), pos, pygame.Rect(ix*5, iy*8, 5, 8))
		pos = (pos[0]+5, pos[1])

class Section:
	def __init__(self):
		self.to_be_removed = False
	def on_draw(self, screen):
		pass
	def on_pygame_event(self, event):
		pass
	def on_tick(self):
		pass

class SectionMgr:
	def __init__(self):
		self.stack = []
	def add(self, section):
		self.stack.append(section)
	def remote(self, section):
		self.stack.remove(section)
	def on_draw(self, screen):
		for section in reversed(self.stack):
			if section.on_draw(screen):
				break
	def on_pygame_event(self, event):
		for section in reversed(self.stack):
			if section.on_pygame_event(event):
				break
	def on_tick(self):
		for section in self.stack:
			if section.to_be_removed:
				self.stack.remove(section)
				continue
			if section.on_tick():
				break

class InfoSection(Section):
	def __init__(self):
		Section.__init__(self)
	def on_draw(self, screen):
		text = "INFORMATION"
		draw_text(screen, (SCREEN_W/2-5*len(text)//2, 32), text)
		draw_text(screen, (32, 60+ 0), "INVA SPACERS FOR LD24 BY CELERON55")
		draw_text(screen, (32, 60+ 8), "THEME: EVOLUTION")

		draw_text(screen, (32, 60+24), "YOUR GOAL IS TO SHOOT THE INVADING")
		draw_text(screen, (32, 60+32), "ALIENS. AIM FOR THE HIGHEST SCORE.")

		draw_text(screen, (32, 60+48), "THE ALIENS EVOLVE BASED ON HOW WELL")
		draw_text(screen, (32, 60+56), "THEY HIT YOU AND HOW LONG THEY STAY")
		draw_text(screen, (32, 60+64), "ALIVE. USE THIS TO YOUR ADVANTAGE.")
		
		screen.blit(get_image("game"), (96, 128+16), alien2_src_rects[0])
		screen.blit(get_image("game"), (SCREEN_W-16-96, 128+16), alien3_src_rects[0])
		return True
	def on_pygame_event(self, event):
		if event.type == pygame.QUIT:
			do_exit = True
		if event.type == pygame.KEYDOWN:
			self.to_be_removed = True
			return True

class ControlsSection(Section):
	def __init__(self):
		Section.__init__(self)
	def on_draw(self, screen):
		text = "CONTROLS"
		draw_text(screen, (SCREEN_W/2-5*len(text)//2, 32), text)
		draw_text(screen, (32, 72+ 0), " MOVE: [LEFT]")
		draw_text(screen, (32, 72+ 8), "       [RIGHT]")
		draw_text(screen, (32, 72+16), "SHOOT: [SPACE]")
		
		screen.blit(get_image("game"), (96, 128+16), alien3_src_rects[0])
		screen.blit(get_image("game"), (SCREEN_W-16-96, 128+16), ship_src_rect)
		return True
	def on_pygame_event(self, event):
		if event.type == pygame.QUIT:
			do_exit = True
		if event.type == pygame.KEYDOWN:
			self.to_be_removed = True
			return True

class MenuSection(Section):
	def __init__(self):
		Section.__init__(self)
	def on_draw(self, screen):
		text = "INVA SPACERS"
		draw_text(screen, (SCREEN_W/2-5*len(text)//2, 32), text)
		text = "[SPACE] START GAME"
		draw_text(screen, (72, 88), text)
		text = "    [I] INFORMATION"
		draw_text(screen, (72, 88+8), text)
		text = "    [C] CONTROLS"
		draw_text(screen, (72, 88+16), text)
		text = "  [ESC] QUIT"
		draw_text(screen, (72, 88+24), text)
		screen.blit(get_image("game"), (96, 56), ship_src_rect)
		screen.blit(get_image("game"), (SCREEN_W-16-96, 56), alien1_src_rects[0])
		return True
	def on_pygame_event(self, event):
		if event.type == pygame.QUIT:
			do_exit = True
		if event.type == pygame.KEYDOWN:
			if event.key == pygame.K_ESCAPE or event.key == pygame.K_q:
				self.to_be_removed = True
			if event.key == pygame.K_SPACE or event.key == pygame.K_RETURN:
				sectionmgr.add(GameSection(Game()))
			if event.key == pygame.K_i:
				sectionmgr.add(InfoSection())
			if event.key == pygame.K_c:
				sectionmgr.add(ControlsSection())
			return True

class AlienLaser:
	def __init__(self, p, speed, origin_alien):
		self.p = p
		self.speed = speed
		self.origin_alien = origin_alien

class AlienProperties:
	def __init__(self):
		self.shoots_initially = False
		self.reload_time = 8000
		self.projectile_speed = 2
		self.first_hit_fly = False
		self.first_hit_explode_bullets = False
		self.fly_mode_adds_weapon = False
		self.fly_range = 4
	
	def get_score(self):
		score = 10
		if self.shoots_initially or self.fly_mode_adds_weapon:
			score += 2 * self.get_reload_time_small_number() * self.projectile_speed
		if self.first_hit_explode_bullets:
			score += 4 * self.get_reload_time_small_number()
		elif self.first_hit_fly:
			score += 4 * self.fly_range
		return score
	
	def get_reload_time_small_number(self):
		if self.reload_time < 300:
			return 12
		elif self.reload_time < 800:
			return 9
		elif self.reload_time < 3000:
			return 6
		elif self.reload_time < 6000:
			return 4
		elif self.reload_time < 9000:
			return 3
		elif self.reload_time < 12000:
			return 2
		else:
			return 1
	
	def mix(self, other):
		new = AlienProperties()
		new.shoots_initially = other.shoots_initially if random.getrandbits(1) else self.shoots_initially
		if random.randint(1,10) == 1:
			new.shoots_initially = not new.shoots_initially
		r = random.random()
		new.reload_time = int(0.5 + self.reload_time*r + other.reload_time*(1-r))
		new.reload_time *= 1.0 + 0.2 * (random.random()*2.0-1.0)
		new.reload_time = int(new.reload_time + 0.5)
		r = random.random()
		new.projectile_speed = int(0.5 + self.projectile_speed*r + other.projectile_speed*(1-r))
		new.projectile_speed *= 1.0 + 0.5 * (random.random()*2.0-1.0)
		new.projectile_speed = int(new.projectile_speed + 0.5)
		new.first_hit_fly = other.first_hit_fly if random.getrandbits(1) else self.first_hit_fly
		if random.randint(1,10) == 1:
			new.first_hit_fly = not new.first_hit_fly
		new.first_hit_explode_bullets = other.first_hit_explode_bullets if random.getrandbits(1) else self.first_hit_explode_bullets
		if random.randint(1,10) == 1:
			new.first_hit_explode_bullets = not new.first_hit_explode_bullets
		new.fly_mode_adds_weapon = other.fly_mode_adds_weapon if random.getrandbits(1) else self.fly_mode_adds_weapon
		if random.randint(1,10) == 1:
			new.fly_mode_adds_weapon = not new.fly_mode_adds_weapon
		r = random.random()
		new.fly_range = int(0.5 + self.fly_range*r + other.fly_range*(1-r))
		new.fly_range *= 1.0 + 0.5 * (random.random()*2.0-1.0)
		new.fly_range = int(new.fly_range + 0.5)
		return new
	
	def dump(self):
		s = ""
		s += "shoots_initially="+str(self.shoots_initially)
		s += ", reload_time="+str(self.reload_time)
		s += ", projectile_speed="+str(self.projectile_speed)
		s += ", first_hit_fly="+str(self.first_hit_fly)
		s += ", fly_mode_adds_weapon="+str(self.fly_mode_adds_weapon)
		s += ", first_hit_explode_bullets="+str(self.first_hit_explode_bullets)
		s += ", fly_range="+str(self.fly_range)
		return s

class Alien:
	def __init__(self, p, properties):
		self.properties = copy.deepcopy(properties)
		self.to_be_removed = False
		self.p = p
		self.initial_p = p
		self.anim_frame = 0
		self.shoots = self.properties.shoots_initially
		self.shoot_timer = -1
		self.fly_mode = False
		self.fly_mode_counter = 0
		self.flat_move_count = 0

		self.die_order = None
	def get_src_rects(self):
		if self.properties.first_hit_explode_bullets:
			return alien3_src_rects
		elif self.properties.first_hit_fly:
			return alien2_src_rects
		else:
			return alien1_src_rects
	def get_src_rect(self):
		return self.get_src_rects()[self.anim_frame]
	def hit(self, game):
		if self.properties.first_hit_explode_bullets:
			d = self.properties.get_reload_time_small_number()
			for dx in range(-d//2, -d//2+d):
				p = (self.p[0] + dx*8 + 8, self.p[1])
				game.alien_lasers.append(AlienLaser(p, self.properties.projectile_speed, self))
			self.to_be_removed = True
			return True
		if self.properties.first_hit_fly and not self.fly_mode:
			self.fly_mode = True
			if self.shoot_timer > 0:
				self.shoot_timer //= 2
			if self.properties.fly_mode_adds_weapon:
				self.shoots = True
			return False
		self.to_be_removed = True
		return True
	def tick(self, game, move_dir, move_down):
		if self.fly_mode:
			flymoves = [
				(2,0),
				(2,-1),
				(1,-1),
				(0,-1),
				(-1,-1),
				(-2,-1),
				(-2,-0),
				(-2,-0),
				(-2,1),
				(-1,1),
				(0,1),
				(1,1),
				(2,1),
				(2,0),
			]
			fr = self.properties.fly_range
			self.p = (self.p[0] + flymoves[self.fly_mode_counter//fr][0],
					self.p[1] + flymoves[self.fly_mode_counter//fr][1])
			self.fly_mode_counter += 1
			if self.fly_mode_counter >= len(flymoves)*fr:
				self.fly_mode_counter = 0
		if move_dir != 0:
			self.p = (self.p[0] + move_dir*2, self.p[1])
			if move_down:
				self.p = (self.p[0], self.p[1] + 8)
				self.flat_move_count = 0
			self.anim_frame += 1
			if self.anim_frame >= len(self.get_src_rects()):
				self.anim_frame = 0
			self.flat_move_count += 1
		if self.shoot_timer > 0 and self.flat_move_count >= 3:
			self.shoot_timer -= 1
			if self.shoot_timer <= 0:
				game.alien_lasers.append(AlienLaser(self.p, self.properties.projectile_speed, self))
		if self.shoot_timer <= 0 and self.shoots:
			self.shoot_timer = random.randint(self.properties.reload_time//2//TICK_LENGTH, self.properties.reload_time*2//TICK_LENGTH)
			if self.fly_mode:
				self.shoot_timer //= 4

class Explosion:
	def __init__(self, p):
		self.p = p
		self.timer = 0

# Initially have easy aliens
initial_alien_properties = []
for i in range(0,22):
	properties = AlienProperties()
	initial_alien_properties.append(properties)

class Game:
	def __init__(self, some_alien_properties=[], initial_score=0, initial_extra_lives=6):
		self.ship_x = SCREEN_W/2
		self.extra_lives = initial_extra_lives
		self.score = initial_score
		self.ship_laser_pos = []
		self.explosions = []
		self.aliens = []
		self.aliens_killed = []
		self.player_hits = [] # List of aliens
		self.aliens_move_i = 0
		self.aliens_move_dir = -1
		self.aliens_move_down = False
		self.aliens_move_dir_count = 4
		self.all_aliens_killed = False
		self.aliens_invaded = False
		self.alien_lasers = []
		self.ticks_from_last_ship_hit = 100000
		self.extra_lives_added_timer = 0
		
		alien_positions = []
		for y in range(0,3):
			for x in range(0,10):
				alien_positions.append((32+x*19, 64-y*16))
		#for y in range(0,2):
		#	for x in range(0,4):
		#		alien_positions.append((32+x*19, 64-y*16))
		#alien_positions.append((32+5*19, 64-1*16))
		
		if len(some_alien_properties) == 0:
			some_alien_properties = initial_alien_properties

		# Create correct amount of alien properties and suffle them
		alien_properties = []
		alien_properties_i = 0
		for _ in alien_positions:
			if alien_properties_i < len(some_alien_properties):
				properties = some_alien_properties[alien_properties_i]
			else:
				properties = AlienProperties()
				properties.shoots_initially = (random.randint(1,2) == 1)
				properties.reload_time = random.randint(10,16000)
				if random.randint(1,3) == 1:
					properties.projectile_speed = random.randint(1,8)
				else:
					properties.projectile_speed = random.randint(1,2)
				properties.first_hit_fly = (random.randint(1,6) == 1)
				properties.first_hit_explode_bullets = (random.randint(1,8) == 1)
				properties.fly_mode_adds_weapon = (random.randint(1,3) == 1)
				if random.randint(1,3) == 1:
					properties.fly_range = random.randint(1,6)
				else:
					properties.fly_range = random.randint(1,3)
			alien_properties.append(properties)
			alien_properties_i += 1
		random.shuffle(alien_properties)

		# Create aliens to determined positions with shuffled properties
		i = 0
		for p in alien_positions:
			properties = alien_properties[i]
			alien = Alien(p, properties)
			self.aliens.append(alien)
			i += 1
	
	def get_leftmost_alien_x(self):
		leftmost_x = None
		for alien in self.aliens:
			if leftmost_x is None or alien.p[0] < leftmost_x:
				leftmost_x = alien.p[0]
		return leftmost_x

	def get_rightmost_alien_x(self):
		rightmost_x = None
		for alien in self.aliens:
			if rightmost_x is None or alien.p[0] > rightmost_x:
				rightmost_x = alien.p[0]
		return rightmost_x

	def tick(self):
		self.score_was = self.score
		self.ticks_from_last_ship_hit += 1
		self.extra_lives_added_timer -= 1
		if self.ticks_from_last_ship_hit < 500//TICK_LENGTH:
			return
		old_laser_pos = self.ship_laser_pos
		self.ship_laser_pos = []
		aliens_shot = []
		for laser_p in old_laser_pos:
			if laser_p[1] < 0:
				continue
			collided = False
			for alien in self.aliens:
				if abs(alien.p[0] - laser_p[0]) < 8 and abs(alien.p[1] - laser_p[1]) < 8:
					died = alien.hit(self)
					if died:
						aliens_shot.append(alien)
						self.score += alien.properties.get_score()
					self.explosions.append(Explosion(alien.p))
					collided = True
					break
			if collided:
				continue;
			self.ship_laser_pos.append((laser_p[0], laser_p[1]-4))
		if len(aliens_shot) == len(self.aliens):
			self.all_aliens_killed = True
		else:
			for alien in aliens_shot:
				alien.to_be_removed = True
		for alien in self.aliens:
			if alien.to_be_removed:
				alien.die_order = len(self.aliens_killed)
				self.aliens_killed.append(alien)
				self.aliens.remove(alien)
		i = 0
		for alien in self.aliens:
			move_dir = 0
			if self.aliens_move_i == i:
				move_dir = self.aliens_move_dir
			alien.tick(self, move_dir, self.aliens_move_down)
			i += 1
		self.aliens_move_i += 1
		if self.aliens_move_i >= len(self.aliens):
			self.aliens_move_down = False
			self.aliens_move_i = 0
			self.aliens_move_dir_count += 1
			if self.aliens_move_dir == -1:
				x = self.get_leftmost_alien_x()
				if x and x < 16+8:
					self.aliens_move_dir = 1
					self.aliens_move_down = True
			else:
				x = self.get_rightmost_alien_x()
				if x is not None and x > SCREEN_W-16-8+1:
					self.aliens_move_dir = -1
					self.aliens_move_down = True
		for e in self.explosions:
			e.timer += 1
			if e.timer >= 600/TICK_LENGTH:
				self.explosions.remove(e)
		for alien in self.aliens:
			if alien.p[1] > SCREEN_H - 32:
				self.aliens_invaded = True
		for alien_laser in self.alien_lasers:
			alien_laser.p = (alien_laser.p[0], alien_laser.p[1] + alien_laser.speed)
			if alien_laser.p[1] > SCREEN_H:
				self.alien_lasers.remove(alien_laser)
			if abs(alien_laser.p[0] - self.ship_x + 1) < 8 and abs(alien_laser.p[1] - (SCREEN_H-24)) < 5:
				self.player_hits.append(alien_laser.origin_alien)
				self.alien_lasers.remove(alien_laser)
				self.explosions.append(Explosion(alien_laser.p))
				self.ticks_from_last_ship_hit = 0
				if self.extra_lives > 0:
					self.extra_lives -= 1
				else:
					self.aliens_invaded = True
		if self.score_was < 5000:
			extra_interval = 1000
		else:
			extra_interval = 5000
		if self.score // extra_interval > self.score_was // extra_interval:
			add_extra_lives = self.score // extra_interval - self.score_was // extra_interval
			self.extra_lives += add_extra_lives
			self.extra_lives_added_timer = 1000/TICK_LENGTH

class GameSection(Section):
	def __init__(self, game=None):
		Section.__init__(self)
		self.game = game or Game()
		self.end_effect_timer = None
		self.start_effect_timer = 0
		self.won = False
	def on_draw(self, screen):
		game = self.game
		
		show_score = True
		show_extra_ships = True
		show_last_extra_ship = True
		show_ship = True
		show_aliens = True
		show_aliens_num = len(game.aliens)
		show_lasers = True
		show_explosions = True
		if self.end_effect_timer is not None:
			fast_blink = (((self.end_effect_timer//(100/TICK_LENGTH)) % 2) == 0)
			if self.won:
				show_aliens = (fast_blink and self.end_effect_timer < 2000/TICK_LENGTH)
			else:
				show_ship = (fast_blink and self.end_effect_timer < 2000/TICK_LENGTH)
			show_lasers = False
			show_explosions = (not fast_blink and self.end_effect_timer < 2000/TICK_LENGTH)
		if self.start_effect_timer < 1000/TICK_LENGTH:
			show_aliens_num = len(game.aliens) * self.start_effect_timer / (500/TICK_LENGTH)
		if game.extra_lives_added_timer > 0:
			fast_blink = (((game.extra_lives_added_timer//(150/TICK_LENGTH)) % 2) == 0)
			show_score = fast_blink
			show_last_extra_ship = fast_blink

		if show_score:
			draw_text(screen, (4,2), "SCORE: "+str(game.score).zfill(5))
		if show_extra_ships:
			for i in range(0, game.extra_lives):
				if i == game.extra_lives - 1:
					if not show_last_extra_ship:
						break
				screen.blit(get_image("game"), (4+i*10, SCREEN_H-8), ship_small_src_rect)
		if show_ship:
			screen.blit(get_image("game"), (game.ship_x-SHIP_W/2, SCREEN_H-32), ship_src_rect)
		if show_aliens:
			i = 0
			for alien in game.aliens:
				p = alien.p
				screen.blit(get_image("game"), (p[0]-8,p[1]-8), alien.get_src_rect())
				i += 1
				if i >= show_aliens_num:
					break
		if show_lasers:
			for p in game.ship_laser_pos:
				screen.blit(get_image("game"), (p[0]-8,p[1]-8), ship_laser_src_rect)
			for laser in game.alien_lasers:
				p = laser.p
				screen.blit(get_image("game"), (p[0]-8,p[1]-8), alien_laser1_src_rect)
		if show_explosions:
			for e in game.explosions:
				p = e.p
				screen.blit(get_image("game"), (p[0]-8,p[1]-8), explosion_src_rect)
		return True
	def on_pygame_event(self, event):
		game = self.game
		if event.type == pygame.QUIT:
			do_exit = True
			return True
		if event.type == pygame.KEYDOWN:
			if event.key == pygame.K_ESCAPE:
				self.to_be_removed = True
			if event.key == pygame.K_SPACE and len(game.ship_laser_pos) < 2:
				game.ship_laser_pos.append((game.ship_x, SCREEN_H-32))
			return True
		if event.type == pygame.KEYUP:
			return True
	def on_tick(self):
		global sectionmgr
		game = self.game
		if self.start_effect_timer < 1000/TICK_LENGTH:
			self.start_effect_timer += 1
			return
		if self.end_effect_timer is not None:
			self.end_effect_timer += 1
			if self.end_effect_timer >= 3000/TICK_LENGTH:
				self.to_be_removed = True
				sectionmgr.add(GameEndSection(self.won, game))
			return
		keys = pygame.key.get_pressed()
		if game.ticks_from_last_ship_hit >= 500//TICK_LENGTH:
			if keys[pygame.K_LEFT]:
				game.ship_x -= 2
			if keys[pygame.K_RIGHT]:
				game.ship_x += 2
			if game.ship_x < SHIP_W/2:
				game.ship_x = SHIP_W/2
			if game.ship_x > SCREEN_W-SHIP_W/2+1:
				game.ship_x = SCREEN_W-SHIP_W/2+1
		game.tick()
		if self.end_effect_timer is None and game.all_aliens_killed:
			self.end_effect_timer = 0
			self.won = True
		if self.end_effect_timer is None and game.aliens_invaded:
			self.end_effect_timer = 0
			self.won = False

class GameEndSection(Section):
	def __init__(self, won, game):
		Section.__init__(self)
		self.won = won
		self.score = game.score
		self.extra_lives = game.extra_lives
		self.next_alien_properties = []
		self.gene_source_aliens = []
		if self.won:
			aliens = game.aliens_killed + game.aliens
			"""print("Aliens:")
			for alien in aliens:
				s = ""
				s += " [" + alien.properties.dump() + "]"
				s += " [player hits: " + game.player_hits.count(alien) + "]"
				if alien.die_order is not None:
					s += " [die order: " + str(alien.die_order) + "]"
				print("-"+s)"""
			best_shooters = []
			for alien in game.player_hits:
				best_shooters.append(alien) # The ones hitting multiple times get many slots
			# If none of the aliens hit the player, take the one that died last
			if len(best_shooters) == 0:
				for alien in reversed(aliens):
					best_shooters.append(alien)
					break
			if len(aliens) >= 2 and len(best_shooters) >= 1:
				count = 0
				for alien in reversed(aliens):
					# Mix with an alien that has hit the player
					other = best_shooters[random.randint(0,len(best_shooters)-1)]
					if alien not in self.gene_source_aliens:
						self.gene_source_aliens.append(alien)
					if other not in self.gene_source_aliens:
						self.gene_source_aliens.append(other)
					prop = alien.properties.mix(other.properties)
					self.next_alien_properties.append(prop)
					#print("Mixed properties #"+str(count)+": "+prop.dump())
					count += 1
					if count >= len(aliens)*2/3:
						break

	def on_draw(self, screen):
		draw_text(screen, (4,2), "SCORE: "+str(self.score).zfill(5))
		for i in range(0, self.extra_lives):
			screen.blit(get_image("game"), (4+i*10, SCREEN_H-8), ship_small_src_rect)
		if self.won:
			text = "WELL DONE EARTHLING"
			draw_text(screen, (SCREEN_W/2-5*len(text)//2, 64), text)
			text = "THIS TIME YOU WIN"
			draw_text(screen, (SCREEN_W/2-5*len(text)//2, 64+8), text)
			"""aliens_offset_y = 1000
			for alien in self.gene_source_aliens:
				p = alien.initial_p
				if p[1] < aliens_offset_y:
					aliens_offset_y = p[1]
			text = "GENES TAKEN FROM:"
			#draw_text(screen, (16+8, 64+8*5), text)
			draw_text(screen, (16+8, aliens_offset_y-16), text)
			for alien in self.gene_source_aliens:
				p = alien.initial_p
				screen.blit(get_image("game"), (p[0]-8,p[1]-8), alien.get_src_rect())
				#screen.blit(get_image("game"), (p[0]-8,p[1]-aliens_offset_y+64+8*5+24), alien.get_src_rect())
			text = "WELL DONE EARTHLING"
			draw_text(screen, (SCREEN_W/2-5*len(text)//2, 96), text)
			text = "THIS TIME YOU WIN"
			draw_text(screen, (SCREEN_W/2-5*len(text)//2, 96+8), text)"""
		else:
			text = "A HA HA"
			draw_text(screen, (SCREEN_W/2-5*len(text)//2, 64), text)
			text = "YUMMY EARTHLINGS"
			draw_text(screen, (SCREEN_W/2-5*len(text)//2, 64+8), text)
		return True
	def on_pygame_event(self, event):
		if event.type == pygame.KEYDOWN:
			self.to_be_removed = True
			if event.key == pygame.K_ESCAPE:
				return True
			if self.won:
				newgame = Game(self.next_alien_properties, self.score, self.extra_lives)
				sectionmgr.add(GameSection(newgame))
			else:
				pass
				#newgame = Game()
				#sectionmgr.add(GameSection(newgame))
			return True
	def on_tick(self):
		pass

pygame.init()

modes = pygame.display.list_modes()
if modes:
	if fullscreen:
		real_screen = pygame.display.set_mode(modes[0], pygame.FULLSCREEN)
	else:
		real_screen = pygame.display.set_mode((1024, 768))
	#real_screen = pygame.display.set_mode((800, 600))
	#real_screen = pygame.display.set_mode((1000, 600))
	#real_screen = pygame.display.set_mode((1440, 1080))
else:
	real_screen = pygame.display.set_mode((800, 600))

sectionmgr = SectionMgr()

pygame.display.set_caption('ld24')
pygame.mouse.set_visible(0)

#sectionmgr.add(BaseSection())
sectionmgr.add(MenuSection())
#sectionmgr.add(GameSection(Game()))
#sectionmgr.add(GameEndSection(True))

tick_time_buf = 0
ticks = pygame.time.get_ticks()
while sectionmgr.stack and not do_exit:
	for event in pygame.event.get():
		sectionmgr.on_pygame_event(event)
	
	while tick_time_buf >= TICK_LENGTH:
		sectionmgr.on_tick()
		tick_time_buf -= TICK_LENGTH

	screenbuf = pygame.Surface((SCREEN_W, SCREEN_H))
	sectionmgr.on_draw(screenbuf)
	screenbuf = pygame.transform.scale(screenbuf, real_screen.get_size())
	real_screen.blit(screenbuf, (0,0))
	pygame.display.update()
	
	ticks_new = pygame.time.get_ticks()
	loop_time = ticks_new - ticks
	ms_to_wait = TICK_LENGTH - loop_time
	if ms_to_wait > 0:
		pygame.time.wait(int(ms_to_wait))
		loop_time = TICK_LENGTH
	else:
		loop_time = ticks_new - ticks
	ticks = ticks_new
	tick_time_buf += loop_time

pygame.quit()
